function SOURCE_CONFIG() {
    this.db_server = null;
    this.db_name = null;
    this.db_group = null;
    this.db_mask = null;
    this.db_item = null;
}


function SOURCE(interval, srv_sel_id, db_sel_id, group_sel_id, mask_sel_id, item_sel_id, apply_id, opts) {
    this.cfg = new SOURCE_CONFIG;

//    interval.SetSource(this);
    this.interval = interval;

    this.apply_button = document.getElementById(apply_id);
    
    this.source_itemsel = new SELECT(item_sel_id, sourceUpdateItem, this);
    this.source_masksel = new SELECT(mask_sel_id, sourceUpdateMask, this);
    this.source_grsel = new SELECT(group_sel_id, sourceUpdateGroup, this);
    this.source_dbsel = new SELECT(db_sel_id, sourceUpdateDatabase, this);
    this.source_srvsel = new SELECT(srv_sel_id, sourceUpdateServer, this);

    this.source_srvsel.SetupSource(adei.GetSelectService("servers", "list_virtual=1&skip_uncached=1"), "db_server");
    this.source_srvsel.SetupCompletionCallbacks(sourceEnableControls, sourceEnableControls);
    this.source_dbsel.SetupSource(adei.GetSelectService("databases", "skip_uncached=1"), "db_name");
    this.source_dbsel.SetupCompletionCallbacks(sourceEnableControls, sourceEnableControls);
    this.source_grsel.SetupSource(adei.GetSelectService("groups", "skip_uncached=1"), "db_group");
    this.source_grsel.SetupCompletionCallbacks(sourceEnableControls, sourceEnableControls);
    this.source_masksel.SetupSource(adei.GetSelectService("masks"), "db_mask");
    this.source_masksel.SetupCompletionCallbacks(sourceEnableControls, sourceEnableControls);
    this.source_itemsel.SetupSource(adei.GetSelectService("items"), "db_item");
    this.source_itemsel.SetupCompletionCallbacks(sourceEnableControls, sourceEnableControls);

    this.source_srvsel.AddChild(this.source_dbsel);
    this.source_dbsel.AddChild(this.source_grsel);
    this.source_grsel.AddChild(this.source_masksel);
    this.source_grsel.AddChild(this.source_itemsel);
    this.source_grsel.AddChild(interval.source_expsel);
    

    this.config = null;
    this.updater = null;
    
    if (typeof opts == "object") this.opts = opts;
    else this.opts = new Object();
}

SOURCE.prototype.Init = function(updater, opts) {
    if (typeof opts == "undefined") opts = new Object;
    opts.reload = true;
    opts.confirm = true;

    opts.cb_attr = this;
    opts.success_cb = sourceConfirmInitialization;

    this.updater = updater;
    this.interval.SetUpdater(updater);
    updater.RegisterControl(this);
    
    this.Update(opts);
}


function sourceConfirmInitialization(source) {
    source.config.Confirm(source);
}

SOURCE.prototype.AttachConfig = function(config) {
    this.config = config;
    this.interval.SetConfig(config);

    config.Register(this);

//    this.Apply();
}


SOURCE.prototype.ApplyConfig = function(subcall) {
    if (this.config) {
    	this.config.SetSource(
	    this.cfg.db_server, 
	    this.cfg.db_name, 
	    this.cfg.db_group, 
	    (isNaN(this.cfg.db_mask)||(this.cfg.db_mask>=0))?this.cfg.db_mask:this.cfg.db_item
	);

	this.interval.ApplyConfig(true);

	if (typeof subcall == "undefined") {
	    this.config.Save();
	}
    }
}

SOURCE.prototype.ReadConfig = function(opts) {
    if (this.config) {
	if (typeof opts == "undefined") opts = new Object;
	    
	opts.db_server = this.config.cfg.db_server;
	opts.db_name = this.config.cfg.db_name;
	opts.db_group = this.config.cfg.db_group;
	
	if (this.config.cfg.db_mask != null) {
    	    var mask = this.config.cfg.db_mask;
	    if (mask.search(',')>=0) {
		opts.db_mask = mask;
		opts.db_item = 0;
	    } else if (isNaN(mask)) {
		opts.db_mask = mask;
		opts.db_item = 0;
	    } else {
		opts.db_mask = -1;
		opts.db_item = mask;
	    }
	}
	
	this.interval.ReadConfig(opts);
	this.Update(opts);
    }
}


SOURCE.prototype.Apply = function() {
    this.ApplyConfig();
    if (this.updater) this.updater.Update();
}

SOURCE.prototype.RegisterGraph = function(graph) {
    this.graphs.push(graph);
}


SOURCE.prototype.DisableControls = function() {
    if (this.apply_button) this.apply_button.disabled = true;
    this.source_srvsel.Disable();
    this.interval.DisableControls();
}

SOURCE.prototype.EnableControls = function() {
    this.interval.EnableControls();
    this.source_srvsel.Enable();
    if (this.apply_button) this.apply_button.disabled = false;
}


SOURCE.prototype.Update = function(opts) {
    if (opts) {
	if (opts.reload) {
	    this.DisableControls();
	    this.source_srvsel.Update(null, opts, opts?opts.db_server:null);
	} else if ((opts.reset)||(opts.db_server)) {
	    this.UpdateServer(opts.db_server, opts);
	}
    }
}


SOURCE.prototype.UpdateServer = function(srv, opts) {
    if (typeof srv == "undefined") {
	srv = this.source_srvsel.GetValue();
    }

    if ((opts)&&(opts.confirmed)) {
	this.cfg.db_server = srv;
    } else if (((opts)&&(opts.reload))||(srv != this.cfg.db_server)) {
	if (typeof opts == "undefined") opts = new Object;
	opts.reload = true;
	
	this.DisableControls();
	this.source_srvsel.UpdateChilds(null, opts, srv, this.cfg.db_server);
    } else if ((opts)&&((opts.db_name)||(opts.reset))) {
	this.UpdateDatabase(opts.db_name, opts);
    }
}


SOURCE.prototype.UpdateDatabase = function(db, opts) {
    if (typeof db == "undefined") {
	db = this.source_dbsel.GetValue();
    }

    if ((opts)&&(opts.confirmed)) {
	this.cfg.db_name = db;
    } else if (((opts)&&(opts.reload))||(db != this.cfg.db_name)) {
	if (typeof opts == "undefined") opts = new Object;
	opts.reload = true;

	this.DisableControls();
	this.source_srvsel.UpdateChilds("db_server=" + this.cfg.db_server, opts, db, this.cfg.db_name);
    } else if ((opts)&&((opts.db_group)||(opts.reset))) {
	this.UpdateGroup(opts.db_group, opts);
    } 
}

/* Handling of Interval/Window updates. 
    a) INTERVAL value passed - UPDATING
    b) 'keep_window' in global or local options - KEEPING
    c) Group have changed - UPDATING
    d) 'reset' in local options - UPDATING 
    e) IGNORING


*/
SOURCE.prototype.UpdateGroup = function(group, opts) {
    if (typeof group == "undefined") {
	group = this.source_grsel.GetValue();
    }

    if ((opts)&&(opts.confirmed)) {
	this.cfg.db_group = group;
    } else if (((opts)&&(opts.reload))||(group != this.cfg.db_group)) {
	if (typeof opts == "undefined") opts = new Object;
	opts.reload = true;
	
	this.DisableControls();
	this.source_grsel.UpdateChilds("db_server=" + this.cfg.db_server + "&db_name=" + this.cfg.db_name, opts, group, this.cfg.db_group);
    } else if ((opts)&&((opts.db_mask)||(opts.db_item)||(opts.experiment)||(opts.window)||(opts.reset))) {
	if ((opts.db_mask)||(opts.reset)) {
	    this.UpdateMask(opts.db_mask, opts);
	} 

	if ((opts.db_item)||(opts.reset)) {
	    this.UpdateItem(opts.db_item, opts);
	}
	
	if ((opts.experiment)||(opts.window)||(opts.reset)) {
	    this.interval.UpdateExperiment(opts.experiment, opts);
	} 
    }
}

SOURCE.prototype.UpdateMask = function(mask, opts) {
    if (typeof mask == "undefined") {
	mask = this.source_masksel.GetValue();
    }
    
    if ((opts)&&(opts.confirmed)) {
	if (mask == -1) {
	    this.source_itemsel.Show();
	    if (!this.module_on_display) this.fix_hidden = true;

	    if (typeof this.geometry_cb == "function") {
		this.geometry_cb(this.geometry_cbattr);
	    }

	    this.cfg.db_mask = mask;
	} else {
	    if ((this.cfg.db_mask == -1)||(this.cfg.db_mask == null)) {
		if (!this.module_on_display) this.fix_hidden = true;
		this.source_itemsel.Hide();
	    }
		
	    this.cfg.db_mask = mask;
	    
	    if ((typeof opts == "undefined")||((opts)&&(opts.apply))) this.Apply();
	}
    } else if (((opts)&&(opts.reload))||(mask != this.cfg.db_mask)) {
	this.DisableControls();
	this.source_masksel.UpdateChilds("db_server=" + this.cfg.db_server + "&db_name=" + this.cfg.db_name + "&db_group=" + this.cfg.db_group, opts, mask, this.cfg.db_mask);
    }
}


SOURCE.prototype.UpdateItem = function(item, opts) {
    if (typeof item == "undefined") {
	item = this.source_itemsel.GetValue();
    }

    if ((opts)&&(opts.confirmed)) {
        this.cfg.db_item = item;
	if (opts.apply) this.Apply();
    } else if (((opts)&&(opts.reload))||(item != this.cfg.db_item)) {
	if (typeof opts == "undefined") opts = { apply: true };

	this.DisableControls();
	this.source_itemsel.UpdateChilds("db_server=" + this.cfg.db_server + "&db_name=" + this.cfg.db_name + "&db_group=" + this.cfg.db_group, opts, item, this.cfg.db_item);
    }
}


SOURCE.prototype.GetSourceProps = function() {
    return "db_server=" + this.cfg.db_server + "&db_name=" + this.cfg.db_name + "&db_group=" + this.cfg.db_group;
}

SOURCE.prototype.RegisterGeometryCallback = function(cb, cbattr) {
    this.interval.RegisterGeometryCallback(cb, cbattr);
    this.geometry_cb = cb;
    this.geometry_cbattr = cbattr;
}

SOURCE.prototype.FixHidden = function() {
		    // Fix IE bug, preventing unhiden hidden subnodes while within hidden element
    if ((this.fix_hidden)&&(this.module_on_display)) {
	if (isNaN(this.cfg.db_mask)) {
	    this.source_itemsel.Show();
	    this.source_itemsel.Hide();
	} else {
	    this.source_itemsel.Hide();
	    this.source_itemsel.Show();
	}
	this.fix_hidden = false;
    }
    this.interval.FixHidden();
}

SOURCE.prototype.Alter = function (type, direction) {
    var opts = new Object({
    	cb_attr: this,
	success_cb: sourceApply,
	keep_window: true,
	reset: true
    });
    
    switch (type) {
	case 's':
	    if (direction > 0) this.source_srvsel.Next(true);
	    else this.source_srvsel.Prev(true);
	    this.UpdateServer(this.source_srvsel.GetValue(), opts);
	break;
	case 'd':
	    if (direction > 0) this.source_dbsel.Next(true);
	    else this.source_dbsel.Prev(true);
	    this.UpdateDatabase(this.source_dbsel.GetValue(), opts);
	break;
	case 'g':
	    if (direction > 0) this.source_grsel.Next(true);
	    else this.source_grsel.Prev(true);
	    this.UpdateGroup(this.source_grsel.GetValue(), opts);
	break;
	case 'm':
	    var val = this.source_masksel.GetValue();
	    if (parseInt(val) < 0) this.source_masksel.SetIndex(0);
	    else {
	        if (direction > 0) 
		    this.source_masksel.Next(true, function (idx,val) {
			if (parseInt(val) < 0) return true;
			return false;
		    });
		else 
		    this.source_masksel.Prev(true, function (idx,val) {
			if (parseInt(val) < 0) return true;
			return false;
		    });
	    }
	    
	    this.UpdateMask(this.source_masksel.GetValue(), opts);
	break;
	case 'i':
	    var val = this.source_masksel.GetValue();
	    if (parseInt(val) == -1) {
		if (direction > 0) this.source_itemsel.Next(true);
		else this.source_itemsel.Prev(true);
	    } else {
		this.source_itemsel.SetIndex(0);
		
		this.source_masksel.SetValue(-1);
	        this.UpdateMask(this.source_masksel.GetValue(), opts);
	    }
	    
	    this.UpdateItem(this.source_itemsel.GetValue(), opts);
	break;
    }
}

SOURCE.prototype.NextServer = function() {
    this.Alter('s', 1);
}

SOURCE.prototype.PrevServer = function() {
    this.Alter('s', -1);
}

SOURCE.prototype.NextDatabase = function() {
    this.Alter('d', 1);
}

SOURCE.prototype.PrevDatabase = function() {
    this.Alter('d', -1);
}

SOURCE.prototype.NextGroup = function() {
    this.Alter('g', 1);
}

SOURCE.prototype.PrevGroup = function() {
    this.Alter('g', -1);
}

SOURCE.prototype.NextMask = function() {
    this.Alter('m', 1);
}

SOURCE.prototype.PrevMask = function() {
    this.Alter('m', -1);
}

SOURCE.prototype.NextItem = function() {
    this.Alter('i', 1);
}

SOURCE.prototype.PrevItem = function() {
    this.Alter('i', -1);
}



function sourceUpdateServer(src, srv, opts) {
    return src.UpdateServer(srv, opts);
}

function sourceUpdateDatabase(src, db, opts) {
    return src.UpdateDatabase(db, opts);
}

function sourceUpdateGroup(src, group, opts) {
    return src.UpdateGroup(group, opts);
}

function sourceUpdateMask(src, mask, opts) {
    return src.UpdateMask(mask, opts);
}

function sourceUpdateItem(src, item, opts) {
    return src.UpdateItem(item, opts);
}

function sourceEnableControls(src) {
    src.EnableControls();
}

function sourceDisableControls(src) {
    src.DisableControls();
}

function sourceFixHidden(source) {
    source.FixHidden();
}

function sourceApply(source) {
    source.Apply();
}
